;;; 
;;; ************************************************************
;;; VGA control code for the TinyVGA board.
;;; 
;;; 32x20 mono version
;;; 
;;; Arjan Scherpenisse <arjan@scherpenisse.net>, oct 2009
;;; This code is released in the public domain.
;;; ************************************************************
;;; 

        LIST P=16F628A, R=DEC    ; Use the PIC16F628 and decimal system

        #include "p16f628a.inc"  ; Include header file
        
        ;; __config  _XT_OSC & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_ON
        __config  _HS_OSC & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_ON

        cblock  0x20     ; Declare variable addresses starting at 0x20
        hloop, vloop     ; hsync and vsync loop variables
        vloopa, vloopb

	    data0,data1,data2,data3,x0,x1,x2,x3
        
        tmp,datacounter,dataaddr,targetaddr

        row0,row0a,row0b,row0c
        row1,row1a,row1b,row1c
        row2,row2a,row2b,row2c
        row3,row3a,row3b,row3c
        row4,row4a,row4b,row4c
        row5,row5a,row5b,row5c
        row6,row6a,row6b,row6c
        row7,row7a,row7b,row7c
        row8,row8a,row8b,row8c
        row9,row9a,row9b,row9c

        row10,row10a,row10b,row10c
        row11,row11a,row11b,row11c

        row12,row12a,row12b,row12c
        row13,row13a,row13b,row13c
        row14,row14a,row14b,row14c
        row15,row15a,row15b,row15c
        row16,row16a,row16b,row16c
        row17,row17a,row17b,row17c
        row18,row18a,row18b,row18c
        row19,row19a,row19b,row19c

        endc

#define p_hsync PORTB,0
#define p_vsync PORTA,3
	
#define p_red   PORTA,0	
#define p_green PORTA,1
#define p_blue  PORTA,2

        
;;;  -----------
;;; INITIALIZE
;;; -----------


#define BLACK   240 ;; = black
#define RED     241 ;; = red
#define GREEN   242 ;; = green
#define YELLOW  243 ;; = yellow
#define BLUE    244 ;; = blue
#define MAGENTA 245 ;; = magenta
#define CYAN    246 ;; = cyaan
#define WHITE   247 ;; = white

#define BG GREEN
#define FG BLACK


#define OF FG-BG
        
        ORG    0x000            ; Program starts at 0x000

	
        clrf   PORTA           ; initialize port a
        clrf   PORTB           ; initialize port b

        bsf    STATUS,RP0      ; ram bank 1

        clrf   TRISA           ; all pins port a output
        clrf   TRISB           ; all pins port b output

        bcf    STATUS,RP0      ; ram bank 0

#include "pattern.inc"	    
        
	    goto start

;;; ---------
;;; MAIN LOOP
;;; ---------

        

	
start:
	
	
	
;;; VERTICAL SYNC
;;; -------------
;;; visble area = 480 lines
;;; front porch = 10 lines
;;; sync pulse  = 2 lines
;;; back porch  = 33 lines

mainloop:	
        ;; data part 1 (240)

        movlw row0
        call hline_load
        
_v_in0: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in0
	    nop

        movlw row1
        call hline_load
        
_v_in1: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in1
	    nop


        movlw row2
        call hline_load
        
_v_in2: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in2
	    nop

        movlw row3
        call hline_load
        
_v_in3: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in3
	    nop

        movlw row4
        call hline_load
        
_v_in4: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in4
	    nop

        movlw row5
        call hline_load
        
_v_in5: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in5
	    nop

        movlw row6
        call hline_load
        
_v_in6: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in6
	    nop

        movlw row7
        call hline_load
        
_v_in7: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in7
	    nop

        movlw row8
        call hline_load
        
_v_in8: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in8
	    nop

        movlw row9
        call hline_load
        
_v_in9: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in9
	    nop

        movlw row10
        call hline_load
        
_v_in10: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in10
	    nop

        movlw row11
        call hline_load
        
_v_in11: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in11
	    nop

        movlw row12
        call hline_load
        
_v_in12: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in12
	    nop

        movlw row13
        call hline_load
        
_v_in13: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in13
	    nop

        movlw row14
        call hline_load
        
_v_in14: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in14
	    nop

        movlw row15
        call hline_load
        
_v_in15: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in15
	    nop

        movlw row16
        call hline_load
        
_v_in16: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in16
	    nop

        movlw row17
        call hline_load
        
_v_in17: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in17
	    nop

        movlw row18
        call hline_load
        
_v_in18: call hline_data
        nop
        movlw   23
        movwf   vloopb
        decfsz  vloopa,f
        goto _v_in18
	    nop

        movlw row19
        call hline_load
        
_v_in19: call hline_data
        nop
        movlw   23
        movwf   vloopa
        decfsz  vloopb,f
        goto _v_in19
	    nop









;;; start front porch

        
        ;; front porch (10 lines)
        movlw   9
        movwf   vloop

_v_xin10: call hline_empty
        nop
        nop
        nop
        decfsz  vloop,f
        goto _v_xin10
	    nop

        call hline_empty
        nop
        nop
        nop
        nop
        
        ;; sync pulse (2 lines)
        nop
        bcf p_vsync

        call hline_empty
        nop
        nop
        nop
        nop
        nop
        nop

        call hline_empty
        nop
        nop
        bsf p_vsync
        nop
        
        ;; back porch (33 lines)
        movlw   32
        movwf   vloop
_v_xin11: call hline_empty
        nop
        nop
        nop
        decfsz  vloop,f
        goto _v_xin11
	    nop

        call hline_empty
        nop
        nop
        nop
        nop
        
        goto mainloop


        
;;; HORIZONTAL SYNC
;;; ---------------
;;; One horizontal line is 31.7us -- 158 instructions.
;;; Within this horizontal line:
;;; vis area    - 25.42us = 127 instr
;;; front porch -  0.63us = 3 instr
;;; sync pulse  -  3.81us = 19 ins
;;; back porch  -  1.9us  = 9 ins

hline_empty:                    ; line without data (black).
        nop
        nop
        nop
        
        
        ;; visible area + front porch = 130 instr
        movlw   25              ;25*5 = 125
        movwf   hloop           ; + these: 125+2 = 127
_he_inner1
	    nop
	    nop 
        decfsz  hloop,F
        goto    _he_inner1      ; inner loop = 5 instr
        nop
        
        ;; 3 left
        nop
	    nop
        nop
        
        ;; now for the hsync pulse
        bcf     p_hsync
        ;; 17 inbetween

        movlw   3               ;3*5 = 15
        movwf   hloop           ; + these two: 15+2 = 17
_he_inner2 nop
        nop
        decfsz  hloop,f
        goto    _he_inner2          ; inner loop = 5 usec.

        bsf     p_hsync

        ;; back porch, 9instr
        nop
        return


#include "hlines.inc"

;;; LOAD DATA. dataattr register should contain address.
hline_load:

        movwf dataaddr
        movlw BG
	    movwf PORTA        
        
        movlw 8
        movwf datacounter
        movlw data0
        movwf targetaddr
        
loop:
        ;; point fsr to source 
        movf dataaddr,W
        movwf FSR
        ;; put source in tmp
        movf INDF,W
        movwf tmp
        ;; point fsr to dest
        movf targetaddr,W
        movwf FSR
        ;; get tmp
        movf tmp,W
        ;; move to dest
        movwf INDF
        ;; increase pointers
        incf dataaddr,f
        incf targetaddr,f
        
        decfsz datacounter,f
        goto loop
        nop
        
        ;; 108

        ;; 23 to go
        nop
        nop

        movlw   3               ;4*5=20
        movwf   hloop           ; + these two: 25+2 = 22
_hdl_inner
        nop
        nop
        decfsz  hloop,f
        goto    _hdl_inner          ; inner loop = 5 usec.
        nop

        movlw 240
	    movwf PORTA        

        ;; here we should have 131 instructions

        ;; now for the hsync pulse
        bcf     p_hsync
        ;; 17 inbetween

        movlw   3               ;3*5 = 15
        movwf   hloop           ; + these two: 15+2 = 17
_hdl_inner2 nop
        nop
        decfsz  hloop,f
        goto    _hdl_inner2          ; inner loop = 5 usec.

        bsf     p_hsync

        ;; back porch, 9instr
        nop

        nop
        nop
        nop
        nop
        nop
        nop
        
        return

        end

        
